unit -->modulenameScanner;

(* NOTE: Any changes made to this file may be overwritten when it gets regenerated
  from within ParserBuilder *)

interface

uses
  Windows, Messages, SysUtils, Classes;

const
  BlkSize = 16384;
  TAB = #09;
  _LF = #10;
  _CR = #13;
  _EF = #0;
  EF = #0;
  _EL = _CR;
  LineEnds: array[0..2] of char = (#13, #10, #0);
  CRLF = #13#10;

type
  TBufBlock = array [0..BlkSize - 1] of char;
  TCocoBuffer = array [0..1024] of ^TBufBlock;
  TStartTable = array [0..255] of integer;
  TGetCH = procedure(pos: longint; Var retCh : Char) of object;
  TCocoErrorProc = procedure(nr, line, col: integer; pos: integer) of object;

type
  T-->modulenameScanner = class(TObject)
  private
    lastCh,
    ch: char;       (*current input character*)
    curLine: integer;    (*current input line (may be higher than line)*)
    lineStart: longint;    (*start position of current line*)
    apx: longint;    (*length of appendix (CONTEXT phrase)*)
    oldEols: integer;    (*number of _EOLs in a comment*)
    bp, bp0: longint;    (*current position in buf
                           (bp0: position of current token)*)
    LBlkSize: longint;    (*BlkSize*)
    inputLen: longint;    (*source file size*)
    buf: TCocoBuffer;     (*source buffer for low-level access*)
    start: TStartTable; (*start state for every character*)
    CurrentCh: TGetCH;
    {----------------}
    FnextLen: integer;
    Fpos: integer;
    Fcol: integer;
    FnextCol: integer;
    FnextPos: integer;
    Flen: integer;
    FnextLine: integer;
    Fline: integer;
    Fdirectory: string;
    FError: TCocoErrorProc;
    FOutputDirectory: string;
    procedure Err(nr, line, col: integer; pos: longint);
    procedure NextCh;
    function Comment: boolean;
    procedure CapChAt(pos: longint; var retChar : Char);
    procedure DisposeBuf;
  protected
  public
    src: TStringStream;         (*source/list files. To be opened by the main pgm*)
    lst: TStringStream;
    Ferrors: integer;
    constructor Create;
    destructor Destroy; override;
    procedure Get(var sym: integer);
    (* Gets next symbol from source file *)
    procedure GetString(pos: longint; len: integer; var s: string);
    (* Retrieves exact string of max length len from position pos in source file *)
    procedure GetName(pos: longint; len: integer; var s: string);
    (* Retrieves name of symbol of length len at position pos in source file *)
    procedure CharAt(pos: longint; var retChar : Char);
    (* Returns exact character at position pos in source file *)
    procedure _Reset;
    (* Reads and stores source file internally *)
    property Error: TCocoErrorProc read FError write FError;
  published
    { Published declarations }
    property directory: string read Fdirectory write Fdirectory;
    (*of source file*)
    property OutputDirectory: string read FOutputDirectory write FOutputDirectory;
    property line: integer read Fline write Fline;
    (*line and column of current symbol*)
    property col: integer read Fcol write Fcol;
    (*line and column of current symbol*)
    property len: integer read Flen write Flen;      (*length of current symbol*)
    property pos: integer read Fpos write Fpos;
    (*file position of current symbol*)
    property nextLine: integer read FnextLine write FnextLine;
    (*line of lookahead symbol*)
    property nextCol: integer read FnextCol write FnextCol;
    (*column of lookahead symbol*)
    property nextLen: integer read FnextLen write FnextLen;
    (*length of lookahead symbol*)
    property nextPos: integer read FnextPos write FnextPos;
    (*file position of lookahead symbol*)
    property errors: integer read Ferrors write Ferrors;
    (*number of detected errors*)
  end;


implementation

-->unitname

const
  no_Sym = -->unknownsym; (*error token code*)
  (* not only for errors but also for not finished states of scanner analysis *)
  _EOF = #26; (*MS-DOS _EOF*)

Procedure T-->modulenameScanner.CapChAt(pos: longint; var retChar : Char);
var
  ch: char;
begin
  if pos >= inputLen then
  begin
    retChar := EF;
    EXIT;
  end;
  ch := upcase(buf[pos div LBlkSize]^[pos mod LBlkSize]);
  if ch <> _EOF then retChar := ch
  else
    retChar := EF
end;

Procedure T-->modulenameScanner.CharAt(pos: longint; var retChar : Char);
var
  ch: char;
begin
  if pos >= inputLen then
  begin
    retChar := EF;
    EXIT;
  end;
  ch := buf[pos div LBlkSize]^[pos mod LBlkSize];
  if ch <> _EOF then retChar := ch
  else
    retChar := EF
end;

function T-->modulenameScanner.Comment: boolean;
label
  999;
  VAR
    level: INTEGER;
    oldLineStart,startLine : LONGINT;
  BEGIN
    level := 1; oldLineStart := lineStart;
-->commentComment := FALSE;
    999:
end;


procedure T-->modulenameScanner.Err(nr, line, col, pos: integer);
begin
  INC(Ferrors)
end;

procedure T-->modulenameScanner.Get(var sym: integer);
var
  state: integer;

  function Equal(s: string): boolean;
  var
    i: integer;
    q: longint;
    cc : Char; // Current Char returned from modifed TGetCh
  begin
    if nextLen <> Length(s) then
    begin
      Equal := False;
      EXIT
    end;
    i := 1;
    q := bp0;
    while i <= nextLen do
    begin
      CurrentCh(q,cc);
      if cc <> s[i] then
      begin
        Equal := False;
        EXIT
      end;
      INC(i);
      INC(q)
    end;
    Equal := True
  end;

  procedure CheckLiteral;
  Var cc : Char;
  begin
    -->literals
  end;

begin (*Get*)
  -->GetSy1
  pos := nextPos;
  nextPos := bp;
  col := nextCol;
  nextCol := bp - lineStart;
  line := nextLine;
  nextLine := curLine;
  len := nextLen;
  nextLen := 0;
  apx := 0;
  state := start[Ord(ch)];
  bp0 := bp;
  while True do
  begin
    NextCh;
    INC(FnextLen);
    case state of
     -->GetSy2
      else
      begin
        sym := no_Sym;
        EXIT (*NextCh already done*)
      end;
    end
  end
end;


procedure T-->modulenameScanner.GetName(pos, len: integer; var s: string);
var
  i: integer;
  p: longint;
  cc : Char;
begin
  if len > 255 then len := 255;
  p := pos;
  i := 1;
  SetLength(S,Len);
  while i <= len do
  begin
    CurrentCh(p,cc);
    s[i] := cc;
    INC(i);
    INC(p)
  end;
end;


procedure T-->modulenameScanner.GetString(pos, len: integer; var s: string);
var
  i: integer;
  p: longint;
  cc : Char;
begin
  if len > 255 then len := 255;
  p := pos;
  i := 1;
  setLength(S,len);
  while i <= len do
  begin
    CharAt(p,cc);
    s[i] := cc;
    INC(i);
    INC(p)
  end;
end;

procedure T-->modulenameScanner.NextCh;
  (* Return global variable ch *)
  Var cc : Char;
begin
  lastCh := ch;
  INC(bp);
  CurrentCh(bp,cc);
  ch := cc;
  if (ch = _EL) or (ch = _LF) and (lastCh <> _EL) then
  begin
    INC(curLine);
    lineStart := bp
  end
end;


procedure T-->modulenameScanner._Reset;
var
  len: longint;
  i, Read: integer;
  AmtTransferred: Integer;
begin (*assert: src has been opened*)
//  len := FileSize(src);
  Len := src.Size;
  DisposeBuf;
  i := 0;
  inputLen := len;
  while len > LBlkSize do
  begin
    NEW(buf[i]);
    Read := BlkSize;
//    BlockRead(src, buf[i]^, Read,AmtTransferred);
    AmtTransferred := src.Read(buf[i]^, Read);
    len := len - AmtTransferred;
    INC(i)
  end;
  NEW(buf[i]);
  Read := len;
  //AmtTransferred :=
  src.Read(buf[i]^, Read);
  buf[i]^[Read] := EF;
  curLine := 1;
  lineStart := -2;
  bp := -1;
  oldEols := 0;
  apx := 0;
  errors := 0;
  NextCh;
end;

constructor T-->modulenameScanner.Create;
begin
  inherited;
  lst:= TStringStream.Create('');
  src := NIL;
  -->initializations
  Error := Err;
  LBlkSize := BlkSize;
  lastCh := EF;
end;

destructor T-->modulenameScanner.Destroy;
begin
  if (src <> NIL) then
    src.Free;
  lst.Free;
  DisposeBuf;
  inherited;
end;

procedure T-->modulenameScanner.DisposeBuf;
var
  idx: integer;
begin
  for idx := 1024 downto 0 do
  begin
    if buf[idx] <> nil then
      Dispose(buf[idx]);
    buf[idx] := nil;
  end;
end;



end. (* -->modulename *)
