{-----------------------------------------------------------------------------
 Unit Name: pbshrichedit
 Author:    Pat Connors
 Purpose:
   Syntax Highlighting Rich Edit.
-----------------------------------------------------------------------------}


unit pbshrichedit;

interface 

uses windows, messages, sysutils, classes, controls, stdctrls,
  comctrls, editstreamcallbackhandler, RichEdit, dialogs, forms, Graphics;

type
  tpbshrichedit = class(TRichEdit)
  private
    feditstreamcallbackhandler: teditstreamcallbackhandler;
    fhighlighton: boolean;
    FTreeView: TTreeView;
    FPlainTextCopy: string;
    FSemActionCOlor: Integer;
    procedure sethiglighton(const Value: boolean);
    procedure SetTreeView(const Value: TTreeView);
    procedure SetPlainTextCopy(const Value: string);
    procedure SetSemActionCOlor(const Value: Integer);
  protected 
    procedure onselectionaltered(Sender: TObject; var key: char);
    procedure OnKeyPushed(Sender: TObject; var Key: word; Shift: TShiftState);
  public 
    constructor Create(aowner: TComponent); override;
    destructor Destroy; override;
    procedure refreshhighlight;
    //    procedure defaulthandler(var message); override;
    procedure ScrollSelectionIntoView;
    property TreeView: TTreeView read FTreeView write SetTreeView;
  published
    property higlighton: boolean read fhighlighton write sethiglighton;
    property PlainTextCopy: string read FPlainTextCopy write SetPlainTextCopy;
    property SemActionCOlor: Integer read FSemActionCOlor write SetSemActionCOlor;
  end;

implementation 

constructor tpbshrichedit.Create(aowner: TComponent);
begin 
  inherited;
  feditstreamcallbackhandler := teditstreamcallbackhandler.Create;
  feditstreamcallbackhandler.richedit := self;
  //  onkeypress := onselectionaltered;
  //  OnKeyDown := OnKeyPushed;
  scrollbars := ssboth;
  wanttabs := False;
  TabStop := True;
  font.size := 11;
  font.Name := 'Trebuchet MS';
  scrollbars := ssboth;
  ReadOnly := False;
  wordwrap := False;
end;

//procedure tpbshrichedit.defaulthandler(var message);
//begin 
//  case tmessage(message).msg of 
//    wm_paint:
//    begin 
//      if (fhighlighton) then if fneedsrepaint then
//          feditstreamcallbackhandler.applyhighlight;
//      fneedsrepaint := False;
//    end;
//  end;
//  inherited;
//end;

destructor tpbshrichedit.Destroy;
begin 
  feditstreamcallbackhandler.Free;
  inherited;
end;

procedure tpbshrichedit.OnKeyPushed(Sender: TObject; var Key: word;
  Shift: TShiftState);
begin
  if (Key = VK_F1) then Application.HelpJump('ATG Editor');
end;

procedure tpbshrichedit.onselectionaltered(Sender: TObject; var key: char);
begin
  //  if (fhighlighton) then case Ord(key) of
  //      13, 32:
  //      begin
  //        feditstreamcallbackhandler.adjustment := 0;
  //        fneedsrepaint := True;
  //        invalidate;
  //      end;
  //      49:
  //      begin
  //        feditstreamcallbackhandler.adjustment := 0;
  //        fneedsrepaint := True;
  //        invalidate;
  //      end;
  //    end;
end;

procedure tpbshrichedit.refreshhighlight;
begin
  if HandleAllocated then
    if (fhighlighton) then
    begin
      feditstreamcallbackhandler.Text := FPlainTextCopy;
      feditstreamcallbackhandler.SemActionCOlor := FSemActionCOlor;
      feditstreamcallbackhandler.applyhighlight;
    end;
end;

procedure tpbshrichedit.ScrollSelectionIntoView;
var 
  CP: TPoint;
begin
  CP := CaretPos;
  ScrollBy(0, CP.Y * 21);
end;

procedure tpbshrichedit.sethiglighton(const Value: boolean);
begin 
  fhighlighton := Value;
end;

procedure tpbshrichedit.SetPlainTextCopy(const Value: string);
begin
  FPlainTextCopy := Value;
end;

procedure tpbshrichedit.SetSemActionCOlor(const Value: Integer);
begin
  FSemActionCOlor := Value;
end;

procedure tpbshrichedit.SetTreeView(const Value: TTreeView);
begin
  FTreeView := Value;
end;

end.
